# Put your API key in the next line, for example: alma_api_key = 'm9zz65b3e62557c74cf896f0f1bc5fdc3123'
# To get your API key, visit https://developers.exlibrisgroup.com
alma_api_key = ''

# Put a default library code in the next line, for example: library_code = 'Main'
library_code = 'Main'
	


# No need to edit below this line...	
from flask import Flask, request, jsonify, render_template
from urllib2 import Request, urlopen, HTTPError
from urllib import urlencode, quote_plus

import datetime, json, ssl, calendar

# if you see the following error:
# <urlopen error [SSL: CERTIFICATE_VERIFY_FAILED] certificate verify failed (_ssl.c:726)>
# you can uncomment the following line, but be warned, this means the certificate is not being checked
#ssl._create_default_https_context = ssl._create_unverified_context

app = Flask(__name__)

@app.route('/')
def index():
	return render_template('index.html')

@app.route('/hours',methods=['GET'])
def hours():

	libraryCode = request.args.get('libraryCode')
	start_date = request.args.get('start_date')
	end_date = request.args.get('end_date')
	format = request.args.get('format').lower()
	alma_format = 'json'
	
	if libraryCode is None:
		libraryCode = library_code
	
	if start_date is not None and end_date is not None:
		_start = datetime.datetime.strptime(start_date,"%Y-%m-%d")
		_end = datetime.datetime.strptime(end_date,"%Y-%m-%d")
		date_diff = _end - _start
		if _start > _end:
			return "End date is before start date.";
			quit
		elif date_diff.days > 34:
			return "Alma can only return a maximum of 34 days.";
			quit
		else:
			start_date = _start.strftime('%Y-%m-%d')
			end_date = _end.strftime('%Y-%m-%d')
		
	if start_date is None:
		datelist = []
		today = datetime.date.today()
		datelist.append(today);
		start_date = datelist[0]
		end_date = start_date
	
	if end_date is None:
		end_date = start_date
		
	if format is None:
		format = 'today'
	elif format == 'fullcalendar':
		_start = datetime.datetime.strptime(request.args.get('start'),"%Y-%m-%d")
		_start = _start + datetime.timedelta(days=7)
		year = _start.year
		month = _start.month
		start = datetime.datetime(year=year,month=month,day=1)
		last_day = calendar.monthrange(year,month)[1]
		end = start.replace(day=last_day)
		start_date = start.strftime("%Y-%m-%d")
		end_date = end.strftime("%Y-%m-%d")
	elif format == 'xml':
		alma_format = 'xml'
	
	url = 'https://api-na.hosted.exlibrisgroup.com/almaws/v1/conf/libraries/{libraryCode}/open-hours'.replace('{libraryCode}',quote_plus(libraryCode))
	queryParams = '?' + urlencode({ quote_plus('from') : start_date , quote_plus('to') : end_date , quote_plus('apikey') : alma_api_key, quote_plus('format') : alma_format })
	
	exl_request = Request(url + queryParams)
	exl_request.get_method = lambda: 'GET'
	try:
		alma_response = urlopen(exl_request).read()
	except HTTPError as e:
		return e
		quit
	
	parsed_json = json.loads(alma_response)
	
	if format == 'json' or format == 'xml':
		return alma_response
	
	elif format == 'today':
		item = parsed_json["day"][0]
		
		_date = process_day(item,"date")
		_from = process_day(item,"from")
		_to = process_day(item,"to")
		
		return process_hours(_date,_from,_to)
	
	elif format == 'html':
		return_str = "";
		for item in parsed_json["day"]:
			print(item)
			_date = process_day(item,"date")
			_from = process_day(item,"from")
			_to = process_day(item,"to")
			_day  = process_day(item,"day_name")
			_hour = process_hours(_date,_from,_to)
			return_str += '<div class="alma_hours"><div class="alma_hours_library"">' + libraryCode + '</div><div class="alma_hours_date">' + _date + '</div><div class="alma_hours_day">' + _day + '</div><div class="alma_hours_time">' + _hour + '</div></div>'
		
		return return_str	
		
	elif format == 'fullcalendar':
		return_array = []
		for item in parsed_json["day"]:
			tmp = {}
			
			tmp['start'] = process_fullcalendar(item,'start')
			tmp['end'] = process_fullcalendar(item,'end')
			tmp['title'] = process_fullcalendar(item,'title')
			
			return_array.append(tmp) 
		
		return json.dumps(return_array)
		
	else:
		return alma_response
				
				
def process_day(day,key):
	if key == "date":
		return day["date"][:-1]
	elif key == "from":
		try:
			return day["hour"][0]["from"]
		except IndexError:
			return "closed"
	elif key == "to":
		try:
			return day["hour"][0]["to"]
		except IndexError:
			return "closed"
	elif key == "day_of_week":
		return day["day_of_week"]
	elif key == "value_of_week":
		return day["day_of_week"]["value"]
	elif key == "day_name":
		return day["day_of_week"]["desc"]
	elif  key == "desc_of_week":
		return day["day_of_week"]["desc"]
		
def process_hours(_date,_from,_to):
	if _from != 'closed':
		_start_time = _date + ' ' + _from
		_end_time = _date + ' ' + _to
		start_time = datetime.datetime.strptime(_start_time,"%Y-%m-%d %H:%M").strftime("%-I:%M%p")
		end_time = datetime.datetime.strptime(_end_time,"%Y-%m-%d %H:%M").strftime("%-I:%M%p")
		
		return '<div class="alma_hours_open">Open ' + start_time + ' - ' + end_time + '</div>'
	else:
		return '<div class="alma_hours_closed">Closed</div>'
		
def process_fullcalendar(day,field):
	_date = day["date"][:-1]
	_title = '';
	
	try:
		_from = day["hour"][0]["from"]
	except IndexError:
		_from = "00:00:00"
		_title = "Closed"
		
	try:
		_to = day["hour"][0]["to"]
	except IndexError:
		_to = "00:00:00"
	
	if _title != 'Closed':
		_start_time = _date + ' ' + _from
		_end_time = _date + ' ' + _to
		start_time = datetime.datetime.strptime(_start_time,"%Y-%m-%d %H:%M").strftime("%-I:%M%p")
		end_time = datetime.datetime.strptime(_end_time,"%Y-%m-%d %H:%M").strftime("%-I:%M%p")
		_title = "Open " + start_time + ' - ' + end_time
		
	if field=='start':
		return _date + ' ' + _from
	elif field=='end':
		return _date + ' ' + _to
	else:
		return _title
	
if __name__ == '__main__':
	app.run(debug=True, port=5000)